@@ -21,6 +21,9 @@
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
+
+#include "closestream.h"
+
 #include "blkidP.h"
 
 static int save_dev(blkid_dev dev, FILE *file)
@@ -30,8 +33,7 @@ static int save_dev(blkid_dev dev, FILE *file)
 	if (!dev || dev->bid_name[0] != '/')
 		return 0;
 
-	DBG(DEBUG_SAVE,
-	    printf("device %s, type %s\n", dev->bid_name, dev->bid_type ?
+	DBG(SAVE, blkid_debug("device %s, type %s", dev->bid_name, dev->bid_type ?
 		   dev->bid_type : "(null)"));
 
 	fprintf(file, "<device DEVNO=\"0x%04lx\" TIME=\"%ld.%ld\"",
@@ -68,7 +70,7 @@ int blkid_flush_cache(blkid_cache cache)
 
 	if (list_empty(&cache->bic_devs) ||
 	    !(cache->bic_flags & BLKID_BIC_FL_CHANGED)) {
-		DBG(DEBUG_SAVE, printf("skipping cache file write\n"));
+		DBG(SAVE, blkid_debug("skipping cache file write"));
 		return 0;
 	}
 
@@ -81,19 +83,22 @@ int blkid_flush_cache(blkid_cache cache)
 	    BLKID_RUNTIME_DIR "/", sizeof(BLKID_RUNTIME_DIR)) == 0) {
 
 		/* default destination, create the directory if necessary */
-		if (stat(BLKID_RUNTIME_DIR, &st) && errno == ENOENT) {
-
-			mkdir(BLKID_RUNTIME_DIR, S_IWUSR|
-						 S_IRUSR|S_IRGRP|S_IROTH|
-						 S_IXUSR|S_IXGRP|S_IXOTH);
+		if (stat(BLKID_RUNTIME_DIR, &st)
+		    && errno == ENOENT
+		    && mkdir(BLKID_RUNTIME_DIR, S_IWUSR|
+						S_IRUSR|S_IRGRP|S_IROTH|
+						S_IXUSR|S_IXGRP|S_IXOTH) != 0
+		    && errno != EEXIST) {
+			DBG(SAVE, blkid_debug("can't create %s directory for cache file",
+					BLKID_RUNTIME_DIR));
+			return 0;
 		}
 	}
 
 	/* If we can't write to the cache file, then don't even try */
 	if (((ret = stat(filename, &st)) < 0 && errno != ENOENT) ||
 	    (ret == 0 && access(filename, W_OK) < 0)) {
-		DBG(DEBUG_SAVE,
-		    printf("can't write to cache file %s\n", filename));
+		DBG(SAVE, blkid_debug("can't write to cache file %s", filename));
 		return 0;
 	}
 
@@ -108,11 +113,11 @@ int blkid_flush_cache(blkid_cache cache)
 		tmp = malloc(strlen(filename) + 8);
 		if (tmp) {
 			sprintf(tmp, "%s-XXXXXX", filename);
-			fd = mkstemp(tmp);
+			fd = mkostemp(tmp, O_RDWR|O_CREAT|O_EXCL|O_CLOEXEC);
 			if (fd >= 0) {
 				if (fchmod(fd, 0644) != 0)
-					DBG(DEBUG_SAVE,	printf("%s: fchmod failed\n", filename));
-				else if ((file = fdopen(fd, "w")))
+					DBG(SAVE, blkid_debug("%s: fchmod failed", filename));
+				else if ((file = fdopen(fd, "w" UL_CLOEXECSTR)))
 					opened = tmp;
 				if (!file)
 					close(fd);
@@ -121,12 +126,11 @@ int blkid_flush_cache(blkid_cache cache)
 	}
 
 	if (!file) {
-		file = fopen(filename, "w");
+		file = fopen(filename, "w" UL_CLOEXECSTR);
 		opened = filename;
 	}
 
-	DBG(DEBUG_SAVE,
-	    printf("writing cache file %s (really %s)\n",
+	DBG(SAVE, blkid_debug("writing cache file %s (really %s)",
 		   filename, opened));
 
 	if (!file) {
@@ -147,12 +151,13 @@ int blkid_flush_cache(blkid_cache cache)
 		ret = 1;
 	}
 
-	fclose(file);
+	if (close_stream(file) != 0)
+		DBG(SAVE, blkid_debug("write failed: %s", filename));
+
 	if (opened != filename) {
 		if (ret < 0) {
 			unlink(opened);
-			DBG(DEBUG_SAVE,
-			    printf("unlinked temp cache %s\n", opened));
+			DBG(SAVE, blkid_debug("unlinked temp cache %s", opened));
 		} else {
 			char *backup;
 
@@ -161,20 +166,17 @@ int blkid_flush_cache(blkid_cache cache)
 				sprintf(backup, "%s.old", filename);
 				unlink(backup);
 				if (link(filename, backup)) {
-					DBG(DEBUG_SAVE,
-						printf("can't link %s to %s\n",
+					DBG(SAVE, blkid_debug("can't link %s to %s",
 							filename, backup));
 				}
 				free(backup);
 			}
 			if (rename(opened, filename)) {
 				ret = errno;
-				DBG(DEBUG_SAVE,
-					printf("can't rename %s to %s\n",
+				DBG(SAVE, blkid_debug("can't rename %s to %s",
 						opened, filename));
 			} else {
-				DBG(DEBUG_SAVE,
-				    printf("moved temp cache %s\n", opened));
+				DBG(SAVE, blkid_debug("moved temp cache %s", opened));
 			}
 		}
 	}
@@ -192,7 +194,7 @@ int main(int argc, char **argv)
 	blkid_cache cache = NULL;
 	int ret;
 
-	blkid_init_debug(DEBUG_ALL);
+	blkid_init_debug(BLKID_DEBUG_ALL);
 	if (argc > 2) {
 		fprintf(stderr, "Usage: %s [filename]\n"
 			"Test loading/saving a cache (filename)\n", argv[0]);
@@ -208,7 +210,7 @@ int main(int argc, char **argv)
 		fprintf(stderr, "error (%d) probing devices\n", ret);
 		exit(1);
 	}
-	cache->bic_filename = blkid_strdup(argv[1]);
+	cache->bic_filename = strdup(argv[1]);
 
 	if ((ret = blkid_flush_cache(cache)) < 0) {
 		fprintf(stderr, "error (%d) saving cache\n", ret);
