@@ -32,18 +32,17 @@ struct fdisk_context;
 struct fdisk_label;
 struct fdisk_parttype;
 struct fdisk_ask;
+struct tt;
 
 /*
  * Supported partition table types (labels)
  */
 enum fdisk_labeltype {
-	FDISK_DISKLABEL_DOS = 1,
-	FDISK_DISKLABEL_SUN = 2,
-	FDISK_DISKLABEL_SGI = 4,
-	FDISK_DISKLABEL_OSF = 8,
-	FDISK_DISKLABEL_MAC = 16,
-	FDISK_DISKLABEL_GPT = 32,
-	FDISK_DISKLABEL_ANY = -1
+	FDISK_DISKLABEL_DOS = (1 << 1),
+	FDISK_DISKLABEL_SUN = (1 << 2),
+	FDISK_DISKLABEL_SGI = (1 << 3),
+	FDISK_DISKLABEL_BSD = (1 << 4),
+	FDISK_DISKLABEL_GPT = (1 << 5)
 };
 
 enum {
@@ -58,10 +57,16 @@ enum {
 	FDISK_ASKTYPE_WARN,
 	FDISK_ASKTYPE_WARNX,
 	FDISK_ASKTYPE_INFO,
-	FDISK_ASKTYPE_YESNO
+	FDISK_ASKTYPE_YESNO,
+	FDISK_ASKTYPE_TABLE,
+	FDISK_ASKTYPE_STRING
 };
 
-
+/* extra flags for info massages (see fdisk_sinfo() */
+enum {
+	FDISK_INFO_COLON = 1,	/* colorize "foo bar:" prefix in the message */
+	FDISK_INFO_SUCCESS,	/* info after successful action */
+};
 
 /* init.c */
 extern void fdisk_init_debug(int mask);
@@ -78,18 +83,25 @@ extern int fdisk_context_set_ask(struct fdisk_context *cxt,
 
 extern int fdisk_context_assign_device(struct fdisk_context *cxt,
 				const char *fname, int readonly);
+extern int fdisk_context_deassign_device(struct fdisk_context *cxt);
 
 extern struct fdisk_label *fdisk_context_get_label(struct fdisk_context *cxt,
 				const char *name);
+extern int fdisk_context_next_label(struct fdisk_context *cxt, struct fdisk_label **lb);
 
 extern int fdisk_context_switch_label(struct fdisk_context *cxt,
 				const char *name);
 
 extern int fdisk_context_set_unit(struct fdisk_context *cxt, const char *str);
+
+#define PLURAL	0
+#define SINGULAR 1
 extern const char *fdisk_context_get_unit(struct fdisk_context *cxt, int n);
-extern int fdisk_context_use_cylinders(struct fdisk_context *cxt);
+
 extern unsigned int fdisk_context_get_units_per_sector(struct fdisk_context *cxt);
 
+extern int fdisk_context_enable_details(struct fdisk_context *cxt, int enable);
+
 /* parttype.c */
 extern struct fdisk_parttype *fdisk_get_parttype_from_code(struct fdisk_context *cxt,
                                 unsigned int code);
@@ -101,7 +113,12 @@ extern struct fdisk_parttype *fdisk_new_unknown_parttype(unsigned int type, cons
 extern void fdisk_free_parttype(struct fdisk_parttype *type);
 extern size_t fdisk_get_nparttypes(struct fdisk_context *cxt);
 
+extern int fdisk_is_parttype_string(struct fdisk_context *cxt);
+
 /* label.c */
+extern int fdisk_require_geometry(struct fdisk_context *cxt);
+extern int fdisk_missing_geometry(struct fdisk_context *cxt);
+
 extern int fdisk_dev_has_disklabel(struct fdisk_context *cxt);
 
 extern int fdisk_dev_is_disklabel(struct fdisk_context *cxt, enum fdisk_labeltype l);
@@ -110,6 +127,11 @@ extern int fdisk_dev_is_disklabel(struct fdisk_context *cxt, enum fdisk_labeltyp
 extern int fdisk_write_disklabel(struct fdisk_context *cxt);
 extern int fdisk_verify_disklabel(struct fdisk_context *cxt);
 extern int fdisk_create_disklabel(struct fdisk_context *cxt, const char *name);
+extern int fdisk_list_disklabel(struct fdisk_context *cxt);
+extern int fdisk_locate_disklabel(struct fdisk_context *cxt, int n, const char **name, off_t *offset, size_t *size);
+
+extern int fdisk_get_disklabel_id(struct fdisk_context *cxt, char **id);
+extern int fdisk_set_disklabel_id(struct fdisk_context *cxt);
 
 extern int fdisk_add_partition(struct fdisk_context *cxt, struct fdisk_parttype *t);
 extern int fdisk_delete_partition(struct fdisk_context *cxt, size_t partnum);
@@ -121,18 +143,67 @@ extern int fdisk_set_partition_type(struct fdisk_context *cxt, size_t partnum,
 extern void fdisk_label_set_changed(struct fdisk_label *lb, int changed);
 extern int fdisk_label_is_changed(struct fdisk_label *lb);
 
+extern void fdisk_label_set_disabled(struct fdisk_label *lb, int disabled);
+extern int fdisk_label_is_disabled(struct fdisk_label *lb);
+
 extern int fdisk_partition_get_status(struct fdisk_context *cxt, size_t partnum, int *status);
 extern int fdisk_partition_is_used(struct fdisk_context *cxt, size_t partnum);
 extern int fdisk_partition_toggle_flag(struct fdisk_context *cxt, size_t partnum, unsigned long flag);
 
 /* alignment.c */
 extern int fdisk_reset_alignment(struct fdisk_context *cxt);
+extern int fdisk_reset_device_properties(struct fdisk_context *cxt);
 
+extern int fdisk_save_user_geometry(struct fdisk_context *cxt,
+			    unsigned int cylinders,
+			    unsigned int heads,
+			    unsigned int sectors);
+
+extern int fdisk_save_user_sector_size(struct fdisk_context *cxt,
+				unsigned int phy,
+				unsigned int log);
+
+extern int fdisk_has_user_device_properties(struct fdisk_context *cxt);
+
+extern int fdisk_reread_partition_table(struct fdisk_context *cxt);
 
 /* dos.c */
 extern int fdisk_dos_enable_compatible(struct fdisk_label *lb, int enable);
 extern int fdisk_dos_is_compatible(struct fdisk_label *lb);
 
+/* sun.h */
+extern int fdisk_sun_set_alt_cyl(struct fdisk_context *cxt);
+extern int fdisk_sun_set_xcyl(struct fdisk_context *cxt);
+extern int fdisk_sun_set_ilfact(struct fdisk_context *cxt);
+extern int fdisk_sun_set_rspeed(struct fdisk_context *cxt);
+extern int fdisk_sun_set_pcylcount(struct fdisk_context *cxt);
+
+/* bsd.c */
+extern int fdisk_bsd_edit_disklabel(struct fdisk_context *cxt);
+extern int fdisk_bsd_write_bootstrap(struct fdisk_context *cxt);
+extern int fdisk_bsd_link_partition(struct fdisk_context *cxt);
+
+/* sgi.h */
+#define SGI_FLAG_BOOT	1
+#define SGI_FLAG_SWAP	2
+extern int fdisk_sgi_set_bootfile(struct fdisk_context *cxt);
+extern int fdisk_sgi_create_info(struct fdisk_context *cxt);
+
+/* gpt */
+extern int fdisk_gpt_partition_set_uuid(struct fdisk_context *cxt, size_t i);
+extern int fdisk_gpt_partition_set_name(struct fdisk_context *cxt, size_t i);
+
+/* dos.c */
+extern struct dos_partition *fdisk_dos_get_partition(
+				struct fdisk_context *cxt,
+				size_t i);
+
+extern int fdisk_dos_fix_order(struct fdisk_context *cxt);
+extern int fdisk_dos_move_begin(struct fdisk_context *cxt, int i);
+extern int fdisk_dos_list_extended(struct fdisk_context *cxt);
+
+#define DOS_FLAG_ACTIVE	1
+
 /* ask.c */
 #define fdisk_is_ask(a, x) (fdisk_ask_get_type(a) == FDISK_ASKTYPE_ ## x)
 
@@ -143,6 +214,9 @@ extern const char *fdisk_ask_get_query(struct fdisk_ask *ask);
 extern int fdisk_ask_set_query(struct fdisk_ask *ask, const char *str);
 extern int fdisk_ask_get_type(struct fdisk_ask *ask);
 extern int fdisk_ask_set_type(struct fdisk_ask *ask, int type);
+extern int fdisk_ask_set_flags(struct fdisk_ask *ask, unsigned int flags);
+extern unsigned int fdisk_ask_get_flags(struct fdisk_ask *ask);
+
 extern int fdisk_do_ask(struct fdisk_context *cxt, struct fdisk_ask *ask);
 
 extern const char *fdisk_ask_number_get_range(struct fdisk_ask *ask);
@@ -161,7 +235,7 @@ extern uint64_t fdisk_ask_number_get_result(struct fdisk_ask *ask);
 extern int fdisk_ask_number_set_result(struct fdisk_ask *ask, uint64_t result);
 extern int fdisk_ask_number_set_relative(struct fdisk_ask *ask, int relative);
 extern int fdisk_ask_number_is_relative(struct fdisk_ask *ask);
-
+extern int fdisk_ask_number_inchars(struct fdisk_ask *ask);
 
 extern int fdisk_ask_number(struct fdisk_context *cxt,
 		     uintmax_t low,
@@ -170,13 +244,28 @@ extern int fdisk_ask_number(struct fdisk_context *cxt,
 		     const char *query,
 		     uintmax_t *result);
 
+extern int fdisk_ask_string(struct fdisk_context *cxt,
+		     const char *query,
+		     char **result);
+
+extern char *fdisk_ask_string_get_result(struct fdisk_ask *ask);
+extern int fdisk_ask_string_set_result(struct fdisk_ask *ask, char *result);
+
 extern int fdisk_ask_yesno(struct fdisk_context *cxt, const char *query, int *result);
 extern uint64_t fdisk_ask_yesno_get_result(struct fdisk_ask *ask);
 extern int fdisk_ask_yesno_set_result(struct fdisk_ask *ask, uint64_t result);
 
-extern int fdisk_info(struct fdisk_context *cxt, const char *fmt, ...);
-extern int fdisk_warnx(struct fdisk_context *cxt, const char *fmt, ...);
-extern int fdisk_warn(struct fdisk_context *cxt, const char *fmt, ...);
+extern int fdisk_info(struct fdisk_context *cxt, const char *fmt, ...)
+			__attribute__ ((__format__ (__printf__, 2, 3)));
+extern int fdisk_colon(struct fdisk_context *cxt, const char *fmt, ...)
+			__attribute__ ((__format__ (__printf__, 2, 3)));
+extern int fdisk_sinfo(struct fdisk_context *cxt, unsigned int flags, const char *fmt, ...)
+			__attribute__ ((__format__ (__printf__, 3, 4)));
+
+extern int fdisk_warnx(struct fdisk_context *cxt, const char *fmt, ...)
+			__attribute__ ((__format__ (__printf__, 2, 3)));
+extern int fdisk_warn(struct fdisk_context *cxt, const char *fmt, ...)
+			__attribute__ ((__format__ (__printf__, 2, 3)));
 
 extern int fdisk_ask_print_get_errno(struct fdisk_ask *ask);
 extern int fdisk_ask_print_set_errno(struct fdisk_ask *ask, int errnum);
