@@ -163,7 +163,7 @@ static void print_all(struct libmnt_context *cxt, char *pattern, int show_label)
 		free(xsrc);
 	}
 
-	mnt_free_cache(cache);
+	mnt_unref_cache(cache);
 	mnt_free_iter(itr);
 }
 
@@ -641,6 +641,8 @@ static struct libmnt_table *append_fstab(struct libmnt_context *cxt,
 
 		mnt_table_set_parser_errcb(fstab, table_parser_errcb);
 		mnt_context_set_fstab(cxt, fstab);
+
+		mnt_unref_table(fstab);	/* reference is handled by @cxt now */
 	}
 
 	if (mnt_table_parse_fstab(fstab, path))
@@ -682,6 +684,8 @@ static void sanitize_paths(struct libmnt_context *cxt)
 
 static void append_option(struct libmnt_context *cxt, const char *opt)
 {
+	if (opt && (*opt == '=' || *opt == '\'' || *opt == '\"' || isblank(*opt)))
+		errx(MOUNT_EX_USAGE, _("unsupported option format: %s"), opt);
 	if (mnt_context_append_options(cxt, opt))
 		err(MOUNT_EX_SYSERR, _("failed to append option '%s'"), opt);
 }
@@ -988,8 +992,7 @@ int main(int argc, char **argv)
 		 * make a connection between the fstab and the canonicalization
 		 * cache.
 		 */
-		struct libmnt_cache *cache = mnt_context_get_cache(cxt);
-		mnt_table_set_cache(fstab, cache);
+		mnt_table_set_cache(fstab, mnt_context_get_cache(cxt));
 	}
 
 	if (!mnt_context_get_source(cxt) &&
@@ -1077,7 +1080,6 @@ int main(int argc, char **argv)
 		success_message(cxt);
 done:
 	mnt_free_context(cxt);
-	mnt_free_table(fstab);
 	return rc;
 }
 
