@@ -89,13 +89,13 @@ static void __attribute__((__noreturn__)) usage(int rc)
 	FILE *out = rc == EXIT_FAILURE ? stderr : stdout;
 
 	fputs(USAGE_HEADER, out);
-	fprintf(out, _(" %s [options] [file ...]\n"), program_invocation_short_name);
+	fprintf(out, _(" %s [options] [<file>...]\n"), program_invocation_short_name);
 	fputs(USAGE_OPTIONS, out);
 	fputs(_(" -c, --columns <width>    width of output in number of characters\n"), out);
 	fputs(_(" -t, --table              create a table\n"), out);
 	fputs(_(" -s, --separator <string> possible table delimiters\n"), out);
-	fputs(_(" -o, --output-separator <string>\n"), out);
-	fputs(_("                          table output column separator, default is two spaces\n"), out);
+	fputs(_(" -o, --output-separator <string>\n"
+	        "                          columns separator for table output; default is two spaces\n"), out);
 	fputs(_(" -x, --fillrows           fill rows before columns\n"), out);
 	fputs(USAGE_SEPARATOR, out);
 	fputs(USAGE_HELP, out);
@@ -308,7 +308,7 @@ wchar_t *local_wcstok(wchar_t * p, const wchar_t * separator, int greedy,
 static void maketbl(wchar_t **list, int entries, wchar_t *separator, int greedy, wchar_t *colsep)
 {
 	TBL *t;
-	int cnt, i;
+	int cnt;
 	wchar_t *p, **lp;
 	ssize_t *lens;
 	ssize_t maxcols = DEFCOLS, coloff;
@@ -347,8 +347,7 @@ static void maketbl(wchar_t **list, int entries, wchar_t *separator, int greedy,
 	for (t = tbl, cnt = 0; cnt < entries; ++cnt, ++t) {
 		for (coloff = 0; coloff < t->cols - 1; ++coloff) {
 			fputws(t->list[coloff], stdout);
-			for (i = lens[coloff] - t->len[coloff]; i > 0; i--)
-				putwchar(' ');
+			wprintf(L"%*s", lens[coloff] - t->len[coloff], "");
 			fputws(colsep, stdout);
 		}
 		if (coloff < t->cols) {
@@ -392,7 +391,7 @@ static int input(FILE *fp, int *maxlength, wchar_t ***list, int *entries)
 			continue;
 		}
 		lineno++;
-		if (!feof(fp))
+		if (!feof(fp) && p)
 			*p = '\0';
 		len = wcs_width(buf);	/* len = p - buf; */
 		if (*maxlength < len)
